function [PercentilesEVD,MeanEVD,EVD]=ForecastErrorVarDecomp(MA,Shocks,Vars,Horizons,ShockLabels,VarLabels,Percentiles,Graph)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Computes the posterior distribution og the forecast error variance decomposition (FEVD)
%
% INPUT:	MA              (NVars-NShocks-NHorizon-niter)  Posterior sample of MA coefficient matrices.
%           Shocks          (NShocks-by-1)                  Shock indicies
%           Vars            (NVars-by-1)                    Variable indicies
%           Horizons        (NHorizon-by-1)                 Horizons where FEVD are computed
%           ShockLabels     (Cell array)                    Labels for the shocks
%           VarLabels       (Cell array)                    Labels for the variables
%			Percentiles: 	(NPercentiles-by-1) vector with percentiles (the 95% percentile is written as 95)
%           GraphTable      (Scalar)                        0=no graph, =1 graph
%				
% OUTPUT:	PercentilesEVD  (NVars-NShocks-NHorizon-NPerc)  Forecast Error Variance Decomposition
%           EVD             (NHorizon-nVars-nShocks-niter)  Posterior Draws of FEVDs
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and 
%                            Department of Statistics, Stockholm University.
%                            E-mail: mattias.villani@riksbank.se.
%
% FIRST VER:   2005-01-13
% REVISED:     2007-09-15
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch length(Percentiles)
    case 1
        Symbol={'k-'};
    case 3
        Symbol={'b--','k-','b--'};
    case 5
        Symbol={'r--','b--','k-','b--','r--'};
    case 7
        Symbol={'g--','r--','b:','k-','b--','r--','g--'};
    otherwise
        Graph=0;
end

LegendPercentiles=cell(0);
for i=1:floor(length(Percentiles)/2)
    LegendPercentiles{i}=[num2str(100*(1-2*Percentiles(i)/100)),'%'];
end
LegendPercentiles{end+1}='Median';

Horizons=Horizons(:)';
Shocks=Shocks(:)';
Vars=Vars(:)';


if isempty(Vars)
    nVars=size(MA,1);
    Vars=1:nVars;
end
if isempty(Shocks)
    nShocks=nVars;
    Shocks=1:nShocks;
end

if isempty(Horizons)
    nHorizons=size(MA,3);
    Horizons=1:nHorizons;
end

nVars=length(Vars);
niter=size(MA,4);
nShocks=length(Shocks);
nHorizons=length(Horizons);
EVD=zeros(size(MA,3),nVars,nShocks,niter);

for i=1:niter
    ShockCount=0;
    for Shock=Shocks
        ShockCount=ShockCount+1;
        HorCount=0;
        for h=0:nHorizons-1
            HorCount=HorCount+1;
            EVD(h+1,:,ShockCount,i)=(MA(Vars,Shock,h+1,i).^2)';
        end
        EVD(:,:,ShockCount,i)=cumsum(EVD(:,:,ShockCount,i));
    end
    
    sumEVD=sum(EVD(:,:,:,i),3);
    ShockCount=0;
    for Shock=Shocks
        ShockCount=ShockCount+1;
        EVD(:,:,ShockCount,i)=EVD(:,:,ShockCount,i)./sumEVD;
    end
end
EVD=EVD(Horizons+1,:,:,:); % Picking out the specified horizons
NPercentiles=length(Percentiles);
MeanEVD=mean(EVD,4);

EVDOrder2=zeros(niter,nVars,nShocks,nHorizons);
for h=1:nHorizons
    for i=1:niter
        for Shock=1:nShocks
            EVDOrder2(i,:,Shock,h)=EVD(h,:,Shock,i);
        end
    end
end
PercentilesEVD=zeros(nVars,nShocks,nHorizons,NPercentiles);

for p=1:NPercentiles
    for h=1:nHorizons
        Temp=[];
        for Shock=1:nShocks
            Temp=[Temp prctile(EVDOrder2(:,:,Shock,h),Percentiles(p))'];
        end
        PercentilesEVD(:,:,h,p)=Temp;
    end
end

% Plot of the Forecast Error Variance Decomposition
if Graph==1
    ShockLabels=ShockLabels(Shocks);
    VarLabels=VarLabels(Vars);
    FigHandle=figure;
    set(FigHandle,'name','Forecast Error Variance Decomposition')
    
    FEVC=zeros(nHorizons,1);
    k=0;
    for i=1:nVars
        for j=1:nShocks
            k=k+1;
            subplot(nVars,nShocks,k)
            hold on
            for p=1:NPercentiles
                for h=1:nHorizons
                    FEVC(h)=PercentilesEVD(i,j,h,p);
                end
                PlotHandle=plot(Horizons,FEVC,Symbol{p});
                set(PlotHandle,'linewidth',1.5);
            end
            axis tight
            if i==1
                title(ShockLabels{j})
            end
            if j==1
                ylabel(VarLabels{i})
            end
            if i==nVars
                xlabel('Horizon')
            end
            set(gca,'ylim',[0 1],'fontsize',8)
            box off
            if i==1 && j==1
                legend(LegendPercentiles,'location','SouthWest')
            end
        end
    end
end
