function [RMSE,MAE,MAPE,MPE,MSSE,LogDetMSSE,TraceMSSE]=PointPredAccuracy(PredsFixedHorizon,DataFixedHorizon,EvalHorizons,MSSESubsetVars,MSSEScaleMatrix)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE:      Computes the measures of point forecast accuracy. Univariate (RMSE, MAE etc) and 
%               Multivariate (Log determinant MSSE matrix etc)
%
% INPUT:        PredsFixedHorizon
%               DataFixedHorizon
%               EvalHorizons
%               MSSEScaleMatrix
%               DieboldMariano
%
% OUTPUT:       RMSE
%               MAE
%               MAPE
%               MPE
%               LogDetMSSE
%               TraceMSSE
%
% AUTHOR:       Mattias Villani, Research Department, Sveriges Riksbank and
%               Department of Statistics, Stockholm University. 
%               E-mail: mattias.villani@riksbank.se
%
% REVISED:      2005-04-27
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% If PredsFixedHorizonNaive is empty, do not try to compute accuracy of naive forecasts
NPredVars=size(PredsFixedHorizon,2);
NModels=size(PredsFixedHorizon,4);
NMSSESubsets=length(MSSESubsetVars);
NEvalHorizons=length(EvalHorizons);
LogDetMSSE=zeros(NEvalHorizons,NMSSESubsets,NModels);
TraceMSSE=zeros(NEvalHorizons,NMSSESubsets,NModels);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Accuracy of point forecasts %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k=1:NModels
    PredError(:,:,:,k)=PredsFixedHorizon(:,:,:,k)-DataFixedHorizon;
end
MPE=zeros(NEvalHorizons,NPredVars,NModels);
RMSE=zeros(NEvalHorizons,NPredVars,NModels);
MAE=zeros(NEvalHorizons,NPredVars,NModels);
MAPE=zeros(NEvalHorizons,NPredVars,NModels);
for k=1:NModels
    for j=1:NEvalHorizons
        AvailableDataObs=1:max(find(PredsFixedHorizon(:,1,j,k)));
        if length(AvailableDataObs)>1
            MPE(j,:,k)=100*mean(PredError(AvailableDataObs,:,j,k)./DataFixedHorizon(AvailableDataObs,:,j));
            RMSE(j,:,k)=sqrt(mean(PredError(AvailableDataObs,:,j,k).^2));
            MAE(j,:,k)=mean(abs(PredError(AvailableDataObs,:,j,k)));    
            MAPE(j,:,k)=100*mean(abs(PredError(AvailableDataObs,:,j,k)./DataFixedHorizon(AvailableDataObs,:,j)));
        else
            MPE(j,:,k)=100*PredError(AvailableDataObs,:,j,k)./DataFixedHorizon(AvailableDataObs,:,j);
            RMSE(j,:,k)=sqrt(PredError(AvailableDataObs,:,j,k).^2);
            MAE(j,:,k)=abs(PredError(AvailableDataObs,:,j,k));
            MAPE(j,:,k)=100*abs(PredError(AvailableDataObs,:,j,k)./DataFixedHorizon(AvailableDataObs,:,j));
        end
        for u=1:NMSSESubsets
            MSSE{u,j,k}=(1/length(AvailableDataObs))*PredError(AvailableDataObs,MSSESubsetVars{u},j,k)'*PredError(AvailableDataObs,MSSESubsetVars{u},j,k);
            LogDetMSSE(j,u,k)=log(det(MSSE{u,j,k}))+log(det(MSSEScaleMatrix(MSSESubsetVars{u},MSSESubsetVars{u})));
            TraceMSSE(j,u,k)=sqrt(trace(MSSEScaleMatrix(MSSESubsetVars{u},MSSESubsetVars{u})*MSSE{u,j,k}));
        end
    end
end

